@@ -54,6 +54,8 @@ logger = logging.getLogger(__name__)
 
 CHAR_STAT_OK = 0
 SERVICE_COMMUNICATION_FAILURE = -70402
+SERVICE_CALLBACK = 0
+SERVICE_CALLBACK_DATA = 1
 
 
 def callback(func):
@@ -131,7 +133,7 @@ class AccessoryDriver:
     def __init__(self, *, address=None, port=51234,
                  persist_file='accessory.state', pincode=None,
                  encoder=None, loader=None, loop=None, mac=None,
-                 listen_address=None, advertised_address=None):
+                 listen_address=None, advertised_address=None, interface_choice=None):
         """
         Initialize a new AccessoryDriver object.
 
@@ -170,6 +172,9 @@ class AccessoryDriver:
             This can be used to announce an external address from behind a NAT.
             If not given, the value of the address parameter will be used.
         :type advertised_address: str
+
+        :param interface_choice: The zeroconf interfaces to listen on.
+        :type InterfacesType: [InterfaceChoice.Default, InterfaceChoice.All]
         """
         if sys.platform == 'win32':
             self.loop = loop or asyncio.ProactorEventLoop()
@@ -185,7 +190,10 @@ class AccessoryDriver:
 
         self.accessory = None
         self.http_server_thread = None
-        self.advertiser = Zeroconf()
+        if interface_choice is not None:
+            self.advertiser = Zeroconf(interfaces=interface_choice)
+        else:
+            self.advertiser = Zeroconf()
         self.persist_file = os.path.expanduser(persist_file)
         self.encoder = encoder or AccessoryEncoder()
         self.topics = {}  # topic: set of (address, port) of subscribed clients
@@ -405,7 +413,7 @@ class AccessoryDriver:
                 if not subscribed_clients:
                     del self.topics[topic]
 
-    def publish(self, data):
+    def publish(self, data, sender_client_addr=None):
         """Publishes an event to the client.
 
         The publishing occurs only if the current client is subscribed to the topic for
@@ -421,7 +429,7 @@ class AccessoryDriver:
 
         data = {HAP_REPR_CHARS: [data]}
         bytedata = json.dumps(data).encode()
-        self.event_queue.put((topic, bytedata))
+        self.event_queue.put((topic, bytedata, sender_client_addr))
 
     def send_events(self):
         """Start sending events from the queue to clients.
@@ -440,10 +448,18 @@ class AccessoryDriver:
         while not self.loop.is_closed():
             # Maybe consider having a pool of worker threads, each performing a send in
             # order to increase throughput.
-            topic, bytedata = self.event_queue.get()
+            #
+            # Clients that made the characteristic change are NOT susposed to get events
+            # about the characteristic change as it can cause an HTTP disconnect and violates
+            # the HAP spec
+            #
+            topic, bytedata, sender_client_addr = self.event_queue.get()
             subscribed_clients = self.topics.get(topic, [])
-            logger.debug('Send event: topic(%s), data(%s)', topic, bytedata)
+            logger.debug('Send event: topic(%s), data(%s), sender_client_addr(%s)', topic, bytedata, sender_client_addr)
             for client_addr in subscribed_clients.copy():
+                if sender_client_addr and sender_client_addr == client_addr:
+                    logger.debug('Skip sending event to client since its the client that made the characteristic change: %s', client_addr)
+                    continue
                 logger.debug('Sending event to client: %s', client_addr)
                 pushed = self.http_server.push_event(bytedata, client_addr)
                 if not pushed:
@@ -625,20 +641,45 @@ class AccessoryDriver:
         :type chars_query: dict
         """
         # TODO: Add support for chars that do no support notifications.
+        service_callbacks = {}
         for cq in chars_query[HAP_REPR_CHARS]:
             aid, iid = cq[HAP_REPR_AID], cq[HAP_REPR_IID]
             char = self.accessory.get_characteristic(aid, iid)
 
             if HAP_PERMISSION_NOTIFY in cq:
                 char_topic = get_topic(aid, iid)
-                logger.debug('Subscribed client %s to topic %s',
-                             client_addr, char_topic)
+                logger.debug(
+                    "Subscribed client %s to topic %s", client_addr, char_topic
+                )
                 self.subscribe_client_topic(
-                    client_addr, char_topic, cq[HAP_PERMISSION_NOTIFY])
+                    client_addr, char_topic, cq[HAP_PERMISSION_NOTIFY]
+                )
 
             if HAP_REPR_VALUE in cq:
                 # TODO: status needs to be based on success of set_value
-                char.client_update_value(cq[HAP_REPR_VALUE])
+                char.client_update_value(cq[HAP_REPR_VALUE], client_addr)
+                # For some services we want to send all the char value
+                # changes at once.  This resolves an issue where we send
+                # ON and then BRIGHTNESS and the light would go to 100%
+                # and then dim to the brightness because each callback
+                # would only send one char at a time.
+                service = char.service
+
+                if service and service.setter_callback:
+                    service_name = service.display_name
+                    service_callbacks.setdefault(aid, {})
+                    service_callbacks[aid].setdefault(
+                        service_name, [service.setter_callback, {}]
+                    )
+                    service_callbacks[aid][service_name][SERVICE_CALLBACK_DATA][
+                        char.display_name
+                    ] = cq[HAP_REPR_VALUE]
+
+        for aid in service_callbacks:
+            for service_name in service_callbacks[aid]:
+                service_callbacks[aid][service_name][SERVICE_CALLBACK](
+                    service_callbacks[aid][service_name][SERVICE_CALLBACK_DATA]
+                )
 
     def signal_handler(self, _signal, _frame):
         """Stops the AccessoryDriver for a given signal.
